

#include <stdio.h>
#include <stdlib.h>
#include <ap_int.h>
#include <ap_utils.h>
#include "dab_prep.h"

typedef struct {
	ap_uint<1> real;
	ap_uint<1> imag;
} cuint1;

typedef struct {
	ap_uint<2> data;
	ap_uint<1> dest;
} cplxui1stream;

typedef struct {
	ap_int<3> real;
	ap_int<3> imag;
} cint3;

typedef struct {
	ap_int<2> real;
	ap_int<2> imag;
} cint2;

cint3 qpsk_sym1to3(cuint1 in)
{
#pragma HLS inline
	cint3 out;

	if(in.real==UI1_MINUS07071){
		out.real = -I3_07071;
	} else {
		out.real = I3_07071;
	}

	if(in.imag==UI1_MINUS07071){
		out.imag = -I3_07071;
	} else {
		out.imag = I3_07071;
	}
	return out;
}

cint3 qpsk_sym1to3v2(ap_uint<2> in)
{
#pragma HLS inline
	cint3 out;

	if(in.bit(0)){
		out.real = I3_07071;
	} else {
		out.real = -I3_07071;
	}

	if(in.bit(1)){
		out.imag = I3_07071;
	} else {
		out.imag = -I3_07071;
	}
	return out;
}




ap_int<3> add3x3(ap_int<3> a, ap_int<3> b)
{
#pragma HLS inline
	/* add zero */
	if(a == I3_ZERO)
		return b;
	if(b == I3_ZERO)
		return a;
	if(a == I3_05 && b == I3_05 )
		return I3_ONE;
	if(a == -I3_05 && b == -I3_05 )
		return -I3_ONE;
	/* self-destroy */
	if(a == -b)
		return I3_ZERO;
	return I3_ZERO;
}

ap_int<3> mul3x3pos(ap_int<3> a, ap_int<3> b)
{
#pragma HLS inline
	/* x*1 = x */
	if(a==I3_ONE)
		return b;
	if(b==I3_ONE)
		return a;

	/* 1,41 * 0,7071 == 1 */
	if(a==I3_07071 && b==I3_07071)
		return I3_05;

	/* x*0 = 0 */
	return I3_ZERO;
}

ap_int<3> mul3x3(ap_int<3> a, ap_int<3> b)
{
#pragma HLS inline
	if(a<0 && b<0)
		return mul3x3pos(-a, -b);
	if(a>=0 && b>=0)
		return mul3x3pos(a, b);
	if(a>=0)
		return -mul3x3pos(a, -b);
	return -mul3x3pos(-a, b);
}



/**************************************************
 * Complex Numbers
 **************************************************
 * (a + bi) * (c + di) = (ac - bd) + (ad + bc)i
 * */

cint3 cmul3x3(cint3 x, cint3 y)
{
//NO!!!
#pragma HLS inline
	cint3 z;
	//-1
	z.real = add3x3(mul3x3(x.real, y.real), -mul3x3(x.imag, y.imag));
	z.imag = add3x3(mul3x3(x.real, y.imag), mul3x3(x.imag, y.real));
#ifdef DEBUG1
	if(dbg) {
		fprintf(stderr, "cmul3x3 out:(%d + %di) * in:(%d + %di) = (%d + %di)\n", (int) x.real, (int) x.imag, (int) y.real, (int) y.imag,
				(int)z.real, (int)z.imag );

		fprintf(stderr, "r: mul3x3(%d, %d) = %d\n", (int) x.real, (int) y.real, (int) mul3x3(x.real, y.real) );
		fprintf(stderr, "i: mul3x3(%d, %d) = %d\n", (int) x.imag, (int) y.imag, (int) mul3x3(x.imag, y.imag) );
		fprintf(stderr, "add3x3(%d, %d) = %d\n", (int) mul3x3(x.real, y.real), (int) -mul3x3(x.imag, y.imag),  (int) add3x3(mul3x3(x.real, y.real), -mul3x3(x.imag, y.imag)));

	}
#endif
	return z;
}













static const cint3 diff_mod_in_phase_ref[1536] = {
{0,-1}, {0,1}, {0,-1}, {0,-1}, {0,-1}, {0,-1}, {1,0}, {1,0}, {0,1}, {0,-1}, {0,-1}, {0,-1}, {0,1}, {0,1}, {1,0}, {1,0},
{0,-1}, {0,1}, {0,-1}, {0,-1}, {0,-1}, {0,-1}, {1,0}, {1,0}, {0,1}, {0,-1}, {0,-1}, {0,-1}, {0,1}, {0,1}, {1,0}, {1,0},
{0,1}, {-1,0}, {0,-1}, {-1,0}, {0,1}, {1,0}, {1,0}, {0,-1}, {0,-1}, {1,0}, {0,-1}, {-1,0}, {0,-1}, {-1,0}, {1,0}, {0,-1},
{0,1}, {-1,0}, {0,-1}, {-1,0}, {0,1}, {1,0}, {1,0}, {0,-1}, {0,-1}, {1,0}, {0,-1}, {-1,0}, {0,-1}, {-1,0}, {1,0}, {0,-1},
{0,1}, {0,1}, {0,1}, {0,-1}, {0,1}, {0,-1}, {-1,0}, {1,0}, {0,-1}, {0,-1}, {0,1}, {0,-1}, {0,-1}, {0,1}, {-1,0}, {1,0},
{0,1}, {0,1}, {0,1}, {0,-1}, {0,1}, {0,-1}, {-1,0}, {1,0}, {0,-1}, {0,-1}, {0,1}, {0,-1}, {0,-1}, {0,1}, {-1,0}, {1,0},
{0,1}, {1,0}, {0,-1}, {1,0}, {0,1}, {-1,0}, {1,0}, {0,1}, {0,-1}, {-1,0}, {0,-1}, {1,0}, {0,-1}, {1,0}, {1,0}, {0,1},
{0,1}, {1,0}, {0,-1}, {1,0}, {0,1}, {-1,0}, {1,0}, {0,1}, {0,-1}, {-1,0}, {0,-1}, {1,0}, {0,-1}, {1,0}, {1,0}, {0,1},
{-1,0}, {1,0}, {-1,0}, {-1,0}, {-1,0}, {-1,0}, {0,-1}, {0,-1}, {1,0}, {-1,0}, {-1,0}, {-1,0}, {1,0}, {1,0}, {0,-1}, {0,-1},
{-1,0}, {1,0}, {-1,0}, {-1,0}, {-1,0}, {-1,0}, {0,-1}, {0,-1}, {1,0}, {-1,0}, {-1,0}, {-1,0}, {1,0}, {1,0}, {0,-1}, {0,-1},
{-1,0}, {0,-1}, {1,0}, {0,-1}, {-1,0}, {0,1}, {0,1}, {1,0}, {1,0}, {0,1}, {1,0}, {0,-1}, {1,0}, {0,-1}, {0,1}, {1,0},
{-1,0}, {0,-1}, {1,0}, {0,-1}, {-1,0}, {0,1}, {0,1}, {1,0}, {1,0}, {0,1}, {1,0}, {0,-1}, {1,0}, {0,-1}, {0,1}, {1,0},
{0,1}, {0,1}, {0,1}, {0,-1}, {0,1}, {0,-1}, {-1,0}, {1,0}, {0,-1}, {0,-1}, {0,1}, {0,-1}, {0,-1}, {0,1}, {-1,0}, {1,0},
{0,1}, {0,1}, {0,1}, {0,-1}, {0,1}, {0,-1}, {-1,0}, {1,0}, {0,-1}, {0,-1}, {0,1}, {0,-1}, {0,-1}, {0,1}, {-1,0}, {1,0},
{1,0}, {0,-1}, {-1,0}, {0,-1}, {1,0}, {0,1}, {0,-1}, {1,0}, {-1,0}, {0,1}, {-1,0}, {0,-1}, {-1,0}, {0,-1}, {0,-1}, {1,0},
{1,0}, {0,-1}, {-1,0}, {0,-1}, {1,0}, {0,1}, {0,-1}, {1,0}, {-1,0}, {0,1}, {-1,0}, {0,-1}, {-1,0}, {0,-1}, {0,-1}, {1,0},
{-1,0}, {1,0}, {-1,0}, {-1,0}, {-1,0}, {-1,0}, {0,-1}, {0,-1}, {1,0}, {-1,0}, {-1,0}, {-1,0}, {1,0}, {1,0}, {0,-1}, {0,-1},
{-1,0}, {1,0}, {-1,0}, {-1,0}, {-1,0}, {-1,0}, {0,-1}, {0,-1}, {1,0}, {-1,0}, {-1,0}, {-1,0}, {1,0}, {1,0}, {0,-1}, {0,-1},
{-1,0}, {0,-1}, {1,0}, {0,-1}, {-1,0}, {0,1}, {0,1}, {1,0}, {1,0}, {0,1}, {1,0}, {0,-1}, {1,0}, {0,-1}, {0,1}, {1,0},
{-1,0}, {0,-1}, {1,0}, {0,-1}, {-1,0}, {0,1}, {0,1}, {1,0}, {1,0}, {0,1}, {1,0}, {0,-1}, {1,0}, {0,-1}, {0,1}, {1,0},
{0,-1}, {0,-1}, {0,-1}, {0,1}, {0,-1}, {0,1}, {1,0}, {-1,0}, {0,1}, {0,1}, {0,-1}, {0,1}, {0,1}, {0,-1}, {1,0}, {-1,0},
{0,-1}, {0,-1}, {0,-1}, {0,1}, {0,-1}, {0,1}, {1,0}, {-1,0}, {0,1}, {0,1}, {0,-1}, {0,1}, {0,1}, {0,-1}, {1,0}, {-1,0},
{0,-1}, {-1,0}, {0,1}, {-1,0}, {0,-1}, {1,0}, {-1,0}, {0,-1}, {0,1}, {1,0}, {0,1}, {-1,0}, {0,1}, {-1,0}, {-1,0}, {0,-1},
{0,-1}, {-1,0}, {0,1}, {-1,0}, {0,-1}, {1,0}, {-1,0}, {0,-1}, {0,1}, {1,0}, {0,1}, {-1,0}, {0,1}, {-1,0}, {-1,0}, {0,-1},
{1,0}, {-1,0}, {1,0}, {1,0}, {1,0}, {1,0}, {0,1}, {0,1}, {-1,0}, {1,0}, {1,0}, {1,0}, {-1,0}, {-1,0}, {0,1}, {0,1},
{1,0}, {-1,0}, {1,0}, {1,0}, {1,0}, {1,0}, {0,1}, {0,1}, {-1,0}, {1,0}, {1,0}, {1,0}, {-1,0}, {-1,0}, {0,1}, {0,1},
{-1,0}, {0,-1}, {1,0}, {0,-1}, {-1,0}, {0,1}, {0,1}, {1,0}, {1,0}, {0,1}, {1,0}, {0,-1}, {1,0}, {0,-1}, {0,1}, {1,0},
{-1,0}, {0,-1}, {1,0}, {0,-1}, {-1,0}, {0,1}, {0,1}, {1,0}, {1,0}, {0,1}, {1,0}, {0,-1}, {1,0}, {0,-1}, {0,1}, {1,0},
{0,1}, {0,1}, {0,1}, {0,-1}, {0,1}, {0,-1}, {-1,0}, {1,0}, {0,-1}, {0,-1}, {0,1}, {0,-1}, {0,-1}, {0,1}, {-1,0}, {1,0},
{0,1}, {0,1}, {0,1}, {0,-1}, {0,1}, {0,-1}, {-1,0}, {1,0}, {0,-1}, {0,-1}, {0,1}, {0,-1}, {0,-1}, {0,1}, {-1,0}, {1,0},
{0,-1}, {-1,0}, {0,1}, {-1,0}, {0,-1}, {1,0}, {-1,0}, {0,-1}, {0,1}, {1,0}, {0,1}, {-1,0}, {0,1}, {-1,0}, {-1,0}, {0,-1},
{0,-1}, {-1,0}, {0,1}, {-1,0}, {0,-1}, {1,0}, {-1,0}, {0,-1}, {0,1}, {1,0}, {0,1}, {-1,0}, {0,1}, {-1,0}, {-1,0}, {0,-1},
{0,-1}, {0,1}, {0,-1}, {0,-1}, {0,-1}, {0,-1}, {1,0}, {1,0}, {0,1}, {0,-1}, {0,-1}, {0,-1}, {0,1}, {0,1}, {1,0}, {1,0},
{0,-1}, {0,1}, {0,-1}, {0,-1}, {0,-1}, {0,-1}, {1,0}, {1,0}, {0,1}, {0,-1}, {0,-1}, {0,-1}, {0,1}, {0,1}, {1,0}, {1,0},
{0,-1}, {1,0}, {0,1}, {1,0}, {0,-1}, {-1,0}, {-1,0}, {0,1}, {0,1}, {-1,0}, {0,1}, {1,0}, {0,1}, {1,0}, {-1,0}, {0,1},
{0,-1}, {1,0}, {0,1}, {1,0}, {0,-1}, {-1,0}, {-1,0}, {0,1}, {0,1}, {-1,0}, {0,1}, {1,0}, {0,1}, {1,0}, {-1,0}, {0,1},
{0,-1}, {0,-1}, {0,-1}, {0,1}, {0,-1}, {0,1}, {1,0}, {-1,0}, {0,1}, {0,1}, {0,-1}, {0,1}, {0,1}, {0,-1}, {1,0}, {-1,0},
{0,-1}, {0,-1}, {0,-1}, {0,1}, {0,-1}, {0,1}, {1,0}, {-1,0}, {0,1}, {0,1}, {0,-1}, {0,1}, {0,1}, {0,-1}, {1,0}, {-1,0},
{1,0}, {0,-1}, {-1,0}, {0,-1}, {1,0}, {0,1}, {0,-1}, {1,0}, {-1,0}, {0,1}, {-1,0}, {0,-1}, {-1,0}, {0,-1}, {0,-1}, {1,0},
{1,0}, {0,-1}, {-1,0}, {0,-1}, {1,0}, {0,1}, {0,-1}, {1,0}, {-1,0}, {0,1}, {-1,0}, {0,-1}, {-1,0}, {0,-1}, {0,-1}, {1,0},
{0,-1}, {0,1}, {0,-1}, {0,-1}, {0,-1}, {0,-1}, {1,0}, {1,0}, {0,1}, {0,-1}, {0,-1}, {0,-1}, {0,1}, {0,1}, {1,0}, {1,0},
{0,-1}, {0,1}, {0,-1}, {0,-1}, {0,-1}, {0,-1}, {1,0}, {1,0}, {0,1}, {0,-1}, {0,-1}, {0,-1}, {0,1}, {0,1}, {1,0}, {1,0},
{1,0}, {0,1}, {-1,0}, {0,1}, {1,0}, {0,-1}, {0,-1}, {-1,0}, {-1,0}, {0,-1}, {-1,0}, {0,1}, {-1,0}, {0,1}, {0,-1}, {-1,0},
{1,0}, {0,1}, {-1,0}, {0,1}, {1,0}, {0,-1}, {0,-1}, {-1,0}, {-1,0}, {0,-1}, {-1,0}, {0,1}, {-1,0}, {0,1}, {0,-1}, {-1,0},
{0,1}, {0,1}, {0,1}, {0,-1}, {0,1}, {0,-1}, {-1,0}, {1,0}, {0,-1}, {0,-1}, {0,1}, {0,-1}, {0,-1}, {0,1}, {-1,0}, {1,0},
{0,1}, {0,1}, {0,1}, {0,-1}, {0,1}, {0,-1}, {-1,0}, {1,0}, {0,-1}, {0,-1}, {0,1}, {0,-1}, {0,-1}, {0,1}, {-1,0}, {1,0},
{0,1}, {1,0}, {0,-1}, {1,0}, {0,1}, {-1,0}, {1,0}, {0,1}, {0,-1}, {-1,0}, {0,-1}, {1,0}, {0,-1}, {1,0}, {1,0}, {0,1},
{0,1}, {1,0}, {0,-1}, {1,0}, {0,1}, {-1,0}, {1,0}, {0,1}, {0,-1}, {-1,0}, {0,-1}, {1,0}, {0,-1}, {1,0}, {1,0}, {0,1},
{0,1}, {0,-1}, {0,1}, {0,1}, {0,1}, {0,1}, {-1,0}, {-1,0}, {0,-1}, {0,1}, {0,1}, {0,1}, {0,-1}, {0,-1}, {-1,0}, {-1,0},
{0,1}, {0,-1}, {0,1}, {0,1}, {0,1}, {0,1}, {-1,0}, {-1,0}, {0,-1}, {0,1}, {0,1}, {0,1}, {0,-1}, {0,-1}, {-1,0}, {-1,0},
{-1,0}, {0,1}, {1,0}, {0,1}, {-1,0}, {0,-1}, {0,1}, {-1,0}, {1,0}, {0,-1}, {1,0}, {0,1}, {1,0}, {0,1}, {0,1}, {-1,0},
{-1,0}, {0,1}, {1,0}, {0,1}, {-1,0}, {0,-1}, {0,1}, {-1,0}, {1,0}, {0,-1}, {1,0}, {0,1}, {1,0}, {0,1}, {0,1}, {-1,0},
{1,0}, {1,0}, {1,0}, {-1,0}, {1,0}, {-1,0}, {0,1}, {0,-1}, {-1,0}, {-1,0}, {1,0}, {-1,0}, {-1,0}, {1,0}, {0,1}, {0,-1},
{1,0}, {1,0}, {1,0}, {-1,0}, {1,0}, {-1,0}, {0,1}, {0,-1}, {-1,0}, {-1,0}, {1,0}, {-1,0}, {-1,0}, {1,0}, {0,1}, {0,-1},
{0,1}, {-1,0}, {0,-1}, {-1,0}, {0,1}, {1,0}, {1,0}, {0,-1}, {0,-1}, {1,0}, {0,-1}, {-1,0}, {0,-1}, {-1,0}, {1,0}, {0,-1},
{0,1}, {-1,0}, {0,-1}, {-1,0}, {0,1}, {1,0}, {1,0}, {0,-1}, {0,-1}, {1,0}, {0,-1}, {-1,0}, {0,-1}, {-1,0}, {1,0}, {0,-1},
{0,-1}, {0,1}, {0,-1}, {0,-1}, {0,-1}, {0,-1}, {1,0}, {1,0}, {0,1}, {0,-1}, {0,-1}, {0,-1}, {0,1}, {0,1}, {1,0}, {1,0},
{0,-1}, {0,1}, {0,-1}, {0,-1}, {0,-1}, {0,-1}, {1,0}, {1,0}, {0,1}, {0,-1}, {0,-1}, {0,-1}, {0,1}, {0,1}, {1,0}, {1,0},
{-1,0}, {0,1}, {1,0}, {0,1}, {-1,0}, {0,-1}, {0,1}, {-1,0}, {1,0}, {0,-1}, {1,0}, {0,1}, {1,0}, {0,1}, {0,1}, {-1,0},
{-1,0}, {0,1}, {1,0}, {0,1}, {-1,0}, {0,-1}, {0,1}, {-1,0}, {1,0}, {0,-1}, {1,0}, {0,1}, {1,0}, {0,1}, {0,1}, {-1,0},
{-1,0}, {-1,0}, {-1,0}, {1,0}, {-1,0}, {1,0}, {0,-1}, {0,1}, {1,0}, {1,0}, {-1,0}, {1,0}, {1,0}, {-1,0}, {0,-1}, {0,1},
{-1,0}, {-1,0}, {-1,0}, {1,0}, {-1,0}, {1,0}, {0,-1}, {0,1}, {1,0}, {1,0}, {-1,0}, {1,0}, {1,0}, {-1,0}, {0,-1}, {0,1},
{0,-1}, {1,0}, {0,1}, {1,0}, {0,-1}, {-1,0}, {-1,0}, {0,1}, {0,1}, {-1,0}, {0,1}, {1,0}, {0,1}, {1,0}, {-1,0}, {0,1},
{0,-1}, {1,0}, {0,1}, {1,0}, {0,-1}, {-1,0}, {-1,0}, {0,1}, {0,1}, {-1,0}, {0,1}, {1,0}, {0,1}, {1,0}, {-1,0}, {0,1},
{-1,0}, {1,0}, {-1,0}, {-1,0}, {-1,0}, {-1,0}, {0,-1}, {0,-1}, {1,0}, {-1,0}, {-1,0}, {-1,0}, {1,0}, {1,0}, {0,-1}, {0,-1},
{-1,0}, {1,0}, {-1,0}, {-1,0}, {-1,0}, {-1,0}, {0,-1}, {0,-1}, {1,0}, {-1,0}, {-1,0}, {-1,0}, {1,0}, {1,0}, {0,-1}, {0,-1},
{0,1}, {1,0}, {0,-1}, {1,0}, {0,1}, {-1,0}, {1,0}, {0,1}, {0,-1}, {-1,0}, {0,-1}, {1,0}, {0,-1}, {1,0}, {1,0}, {0,1},
{0,1}, {1,0}, {0,-1}, {1,0}, {0,1}, {-1,0}, {1,0}, {0,1}, {0,-1}, {-1,0}, {0,-1}, {1,0}, {0,-1}, {1,0}, {1,0}, {0,1},
{-1,0}, {-1,0}, {-1,0}, {1,0}, {-1,0}, {1,0}, {0,-1}, {0,1}, {1,0}, {1,0}, {-1,0}, {1,0}, {1,0}, {-1,0}, {0,-1}, {0,1},
{-1,0}, {-1,0}, {-1,0}, {1,0}, {-1,0}, {1,0}, {0,-1}, {0,1}, {1,0}, {1,0}, {-1,0}, {1,0}, {1,0}, {-1,0}, {0,-1}, {0,1},
{0,-1}, {1,0}, {0,1}, {1,0}, {0,-1}, {-1,0}, {-1,0}, {0,1}, {0,1}, {-1,0}, {0,1}, {1,0}, {0,1}, {1,0}, {-1,0}, {0,1},
{0,-1}, {1,0}, {0,1}, {1,0}, {0,-1}, {-1,0}, {-1,0}, {0,1}, {0,1}, {-1,0}, {0,1}, {1,0}, {0,1}, {1,0}, {-1,0}, {0,1},
{0,1}, {0,-1}, {0,1}, {0,1}, {0,1}, {0,1}, {-1,0}, {-1,0}, {0,-1}, {0,1}, {0,1}, {0,1}, {0,-1}, {0,-1}, {-1,0}, {-1,0},
{0,1}, {0,-1}, {0,1}, {0,1}, {0,1}, {0,1}, {-1,0}, {-1,0}, {0,-1}, {0,1}, {0,1}, {0,1}, {0,-1}, {0,-1}, {-1,0}, {-1,0},
{-1,0}, {0,1}, {1,0}, {0,1}, {-1,0}, {0,-1}, {0,1}, {-1,0}, {1,0}, {0,-1}, {1,0}, {0,1}, {1,0}, {0,1}, {0,1}, {-1,0},
{-1,0}, {0,1}, {1,0}, {0,1}, {-1,0}, {0,-1}, {0,1}, {-1,0}, {1,0}, {0,-1}, {1,0}, {0,1}, {1,0}, {0,1}, {0,1}, {-1,0},
{0,-1}, {0,-1}, {0,-1}, {0,1}, {0,-1}, {0,1}, {1,0}, {-1,0}, {0,1}, {0,1}, {0,-1}, {0,1}, {0,1}, {0,-1}, {1,0}, {-1,0},
{0,-1}, {0,-1}, {0,-1}, {0,1}, {0,-1}, {0,1}, {1,0}, {-1,0}, {0,1}, {0,1}, {0,-1}, {0,1}, {0,1}, {0,-1}, {1,0}, {-1,0},
{0,-1}, {1,0}, {0,1}, {1,0}, {0,-1}, {-1,0}, {-1,0}, {0,1}, {0,1}, {-1,0}, {0,1}, {1,0}, {0,1}, {1,0}, {-1,0}, {0,1},
{0,-1}, {1,0}, {0,1}, {1,0}, {0,-1}, {-1,0}, {-1,0}, {0,1}, {0,1}, {-1,0}, {0,1}, {1,0}, {0,1}, {1,0}, {-1,0}, {0,1},
{-1,0}, {1,0}, {-1,0}, {-1,0}, {-1,0}, {-1,0}, {0,-1}, {0,-1}, {1,0}, {-1,0}, {-1,0}, {-1,0}, {1,0}, {1,0}, {0,-1}, {0,-1},
{-1,0}, {1,0}, {-1,0}, {-1,0}, {-1,0}, {-1,0}, {0,-1}, {0,-1}, {1,0}, {-1,0}, {-1,0}, {-1,0}, {1,0}, {1,0}, {0,-1}, {0,-1},
{-1,0}, {0,1}, {1,0}, {0,1}, {-1,0}, {0,-1}, {0,1}, {-1,0}, {1,0}, {0,-1}, {1,0}, {0,1}, {1,0}, {0,1}, {0,1}, {-1,0},
{-1,0}, {0,1}, {1,0}, {0,1}, {-1,0}, {0,-1}, {0,1}, {-1,0}, {1,0}, {0,-1}, {1,0}, {0,1}, {1,0}, {0,1}, {0,1}, {-1,0},
{-1,0}, {-1,0}, {-1,0}, {1,0}, {-1,0}, {1,0}, {0,-1}, {0,1}, {1,0}, {1,0}, {-1,0}, {1,0}, {1,0}, {-1,0}, {0,-1}, {0,1},
{-1,0}, {-1,0}, {-1,0}, {1,0}, {-1,0}, {1,0}, {0,-1}, {0,1}, {1,0}, {1,0}, {-1,0}, {1,0}, {1,0}, {-1,0}, {0,-1}, {0,1},
{0,1}, {-1,0}, {0,-1}, {-1,0}, {0,1}, {1,0}, {1,0}, {0,-1}, {0,-1}, {1,0}, {0,-1}, {-1,0}, {0,-1}, {-1,0}, {1,0}, {0,-1},
{0,1}, {-1,0}, {0,-1}, {-1,0}, {0,1}, {1,0}, {1,0}, {0,-1}, {0,-1}, {1,0}, {0,-1}, {-1,0}, {0,-1}, {-1,0}, {1,0}, {0,-1},
{0,1}, {0,-1}, {0,1}, {0,1}, {0,1}, {0,1}, {-1,0}, {-1,0}, {0,-1}, {0,1}, {0,1}, {0,1}, {0,-1}, {0,-1}, {-1,0}, {-1,0},
{0,1}, {0,-1}, {0,1}, {0,1}, {0,1}, {0,1}, {-1,0}, {-1,0}, {0,-1}, {0,1}, {0,1}, {0,1}, {0,-1}, {0,-1}, {-1,0}, {-1,0},
{0,-1}, {-1,0}, {0,1}, {-1,0}, {0,-1}, {1,0}, {-1,0}, {0,-1}, {0,1}, {1,0}, {0,1}, {-1,0}, {0,1}, {-1,0}, {-1,0}, {0,-1},
{0,-1}, {-1,0}, {0,1}, {-1,0}, {0,-1}, {1,0}, {-1,0}, {0,-1}, {0,1}, {1,0}, {0,1}, {-1,0}, {0,1}, {-1,0}, {-1,0}, {0,-1},
{0,1}, {0,1}, {0,1}, {0,-1}, {0,1}, {0,-1}, {-1,0}, {1,0}, {0,-1}, {0,-1}, {0,1}, {0,-1}, {0,-1}, {0,1}, {-1,0}, {1,0},
{0,1}, {0,1}, {0,1}, {0,-1}, {0,1}, {0,-1}, {-1,0}, {1,0}, {0,-1}, {0,-1}, {0,1}, {0,-1}, {0,-1}, {0,1}, {-1,0}, {1,0},
{-1,0}, {0,-1}, {1,0}, {0,-1}, {-1,0}, {0,1}, {0,1}, {1,0}, {1,0}, {0,1}, {1,0}, {0,-1}, {1,0}, {0,-1}, {0,1}, {1,0},
{-1,0}, {0,-1}, {1,0}, {0,-1}, {-1,0}, {0,1}, {0,1}, {1,0}, {1,0}, {0,1}, {1,0}, {0,-1}, {1,0}, {0,-1}, {0,1}, {1,0}
};

static cint3 diff_mod_out_phase_buff[1536];

static ap_uint<1> send_tii = 0;

//75-208-123-0-2-0
//69-208-121-0-2-0
//60-188-120-0-2-0
void dqpsk_modulator(hls::stream<ap_uint<3> > &input, hls::stream<ap_uint<6> > &output, const ap_uint<13> comb_pat)
{
#pragma HLS ARRAY_MAP variable=diff_mod_in_phase_ref instance=packed1 vertical
#pragma HLS ARRAY_MAP variable=diff_mod_out_phase_buff instance=packed1 vertical
//#pragma HLS DATAFLOW
//#pragma HLS PIPELINE II=1
#pragma HLS INTERFACE ap_ctrl_none port=return

#pragma HLS INTERFACE ap_none port=comb_pat

//#pragma HLS INTERFACE axis port=input
//#pragma HLS INTERFACE axis port=output
//#pragma HLS RESOURCE variable=input core=AXI4Stream
//#pragma HLS RESOURCE variable=output core=AXI4Stream
//#pragma HLS INTERFACE ap_ctrl_none port=return
#pragma HLS INTERFACE ap_fifo port=input
#pragma HLS INTERFACE ap_fifo port=output
//#pragma HLS RESOURCE variable=input  core=AXIS metadata="-bus_bundle S_AXIS"
//#pragma HLS RESOURCE variable=output core=AXIS metadata="-bus_bundle M_AXIS"

//#pragma HLS INTERFACE axis port=input
//#pragma HLS INTERFACE axis port=output
//#pragma HLS INTERFACE ap_ctrl_none port=return

	ap_uint<11> i;
	//stream_in_data x;
	//ap_uint<6> y;
	ap_uint<3> x;

#if 0
	//100-246-151-0-2-6
	x = input.read(); //data=0 -> -0.7071, data=1 -> 0.7071
    if(x.bit(2)) {
    	for(i=0;i<1536;i++){
    		diff_mod_out_phase_buff[i] = diff_mod_in_phase_ref[i];
    		output.write((diff_mod_out_phase_buff[i].imag.range(2,0), diff_mod_out_phase_buff[i].real.range(2,0)));
//					(diff_mod_out_phase_buff[i].imag.bit(2), diff_mod_out_phase_buff[i].imag.bit(1), diff_mod_out_phase_buff[i].imag.bit(0),
//					 diff_mod_out_phase_buff[i].real.bit(2), diff_mod_out_phase_buff[i].real.bit(1), diff_mod_out_phase_buff[i].real.bit(0))
    	}
    } else {
    	for(i=0;i<1536;i++) {
    		if(i!=0)
    			x = input.read();

//			cuint1 tmp;
//			tmp.imag = x.bit(1);
//			tmp.real = x.bit(0);
			diff_mod_out_phase_buff[i] = cmul3x3(diff_mod_out_phase_buff[i], qpsk_sym1to3v2(x.range(1,0)));

			output.write((diff_mod_out_phase_buff[i].imag.range(2,0), diff_mod_out_phase_buff[i].real.range(2,0)));
//				( diff_mod_out_phase_buff[i].imag.bit(2), diff_mod_out_phase_buff[i].imag.bit(1), diff_mod_out_phase_buff[i].imag.bit(0),
//				  diff_mod_out_phase_buff[i].real.bit(2), diff_mod_out_phase_buff[i].real.bit(1), diff_mod_out_phase_buff[i].real.bit(0))
//				  );
    	}
    }
#endif

#if 0
    //83-200-144-0-2-6
	ap_uint<1> is_prbs = 0;
	for(i=0;i<1536;i++){
		if(i==0 || !is_prbs) {
			x = input.read();
			if(i==0) is_prbs = x.bit(2);
		}
		if(is_prbs) {
			diff_mod_out_phase_buff[i] = diff_mod_in_phase_ref[i];
//					(diff_mod_out_phase_buff[i].imag.bit(2), diff_mod_out_phase_buff[i].imag.bit(1), diff_mod_out_phase_buff[i].imag.bit(0),
//					 diff_mod_out_phase_buff[i].real.bit(2), diff_mod_out_phase_buff[i].real.bit(1), diff_mod_out_phase_buff[i].real.bit(0))
		} else {
			diff_mod_out_phase_buff[i] = cmul3x3(diff_mod_out_phase_buff[i], qpsk_sym1to3v2(x.range(1,0)));


		}
		output.write((diff_mod_out_phase_buff[i].imag.range(2,0), diff_mod_out_phase_buff[i].real.range(2,0)));
	}
#endif



	ap_uint<1> is_null = 0;
	ap_uint<1> is_prbs = 0;

	ap_uint<3> segment_id2 = 0; //overflow is ok
	ap_uint<6> comb_pos2 = 0; // up to 48 only


	for(i=0;i<1536;i++){
		if(i==0 || !is_prbs) {
			x = input.read();
			if(i==0) {
				is_prbs = x.bit(2);
				is_null = x.bit(1);
				if(is_prbs && is_null) {
					send_tii = !send_tii;
				} else {
					is_null = 0;
				}
			}
		}
		if(is_prbs) {
			//b-> is 48 carriers step, so if enabled, then here will be placed carrier with comb offset.
			if(is_null) {
				diff_mod_out_phase_buff[i] = (cint3) {0,0}; // prevent nulling for carriers of future TII marking.
				if(send_tii && comb_pat.get_bit(segment_id2) && (comb_pos2.range(5,1) == comb_pat.range(12,8))) {
					diff_mod_out_phase_buff[i].real = 3 * diff_mod_in_phase_ref[i].real;
					diff_mod_out_phase_buff[i].imag = 3 * diff_mod_in_phase_ref[i].imag;
				}

				if(comb_pos2 < 47)
					comb_pos2++;
				else {
					comb_pos2=0;
					segment_id2++;
				}
			} else {
				diff_mod_out_phase_buff[i] = diff_mod_in_phase_ref[i];
			}
		} else {
			diff_mod_out_phase_buff[i] = cmul3x3(diff_mod_out_phase_buff[i], qpsk_sym1to3v2(x.range(1,0)));
		}
		output.write((diff_mod_out_phase_buff[i].imag.range(2,0), diff_mod_out_phase_buff[i].real.range(2,0)));
	}

}

